##########################################################################################################################
# Auto calibration procedure for Hg water based model                                                                    #
#                                                                                                                        #
# Use the Complex method ('Box method') to calibrate Hg concentration at calibration watershed                           #
#                                                                                                                        #
# Obserevd Hg concentration are smapled from a range for each watershed, and each time the sampled data set is used as   #
# observed data for model calibration.                                                                                   #
#------------------------------------------------------------------------------------------------------------------------#
# By Jing Wu, April 2016                                                                                                 #
##########################################################################################################################

# clear uP the work directory
rm(list=ls()) 

library(ggplot2)
library(plyr) 

###### READ IN INPUT DATA #################################
#setwd('S:/Research/RMP/Small Tribs Loadings Studies/spreadsheet model/Auto_calibration_for_PCB_Hg/');
setwd('C:/Projects/RWSM/2016 Work/Hg/');

### Weighting factor for each observed data point
weightft <- read.csv('Input data/Hg_area_weights.csv', header=TRUE);
weightft <- weightft[order(weightft$Watershed), ]               #sort data to arrange watersheds in alphabetical order
weight <- as.matrix(weightft[,2:ncol(weightft)]); rownames(weight) <- weightft$Watershed; 

### Simulated runoff from each land use within each watershed 
water <- read.csv('Input data/Runoff_at_Hg_Watersheds.csv', header=TRUE);
water <- water[order(water$Watershed), ] 
runoff <- as.matrix(water[,2:ncol(water)]); rownames(runoff) <- water$Watershed; 

### Simulated total runoff from each watershed
watertot <- read.csv('Input data/Total_runoff_at_Hg_watersheds.csv', header=TRUE);
watertot <- watertot[order(watertot$Watershed), ] 
runofftot <- as.matrix(watertot[,2]); rownames(runofftot) <- watertot$Watershed; 

### Initial values & Bounds on contaminent ratio 
concbound <- read.csv('Input data/Hg_concentration_boundary.csv', header=TRUE);
lowbound <- as.numeric(concbound[which(concbound$Conc=="Lowbound"), 2:ncol(concbound)]);
upbound <- as.numeric(concbound[which(concbound$Conc=="Upbound"), 2:ncol(concbound)]);

### read observed data range for each watershed
obHg <- read.csv('Input data/Observed_Hg_concentration.csv', header=TRUE);  
obHg <- obHg[order(obHg$Watershed), ]               

mean <- as.matrix(obHg[,2]); rownames(mean) <- obHg$Watershed;
std <- as.matrix(obHg[,3]); rownames(std) <- obHg$Watershed;
minconc <- as.matrix(obHg[,4]); rownames(minconc) <- obHg$Watershed; 
conc10 <- as.matrix(obHg[,5]); rownames(conc10) <- obHg$Watershed;  
conc25 <- as.matrix(obHg[,6]); rownames(conc25) <- obHg$Watershed;
median <- as.matrix(obHg[,7]); rownames(median) <- obHg$Watershed;
conc75 <- as.matrix(obHg[,8]); rownames(conc75) <- obHg$Watershed;  
conc90 <- as.matrix(obHg[,9]); rownames(conc90) <- obHg$Watershed;
maxconc <- as.matrix(obHg[,10]); rownames(maxconc) <- obHg$Watershed; 

### Get Number of Watershed and Land Use
num_WS <- length(obHg$Watershed); # number of watersheds in input dataset
num_LU <- ncol(concbound)-1;      # number of source area in input dataset     

### Lognormal distribution for observed data 
meanlog =log(mean/(1+std**2/mean**2)^0.5)
sdlog = (log(1+std**2/mean**2))^0.5

### Loop to randomly generate observed data points for model calibration 
num_dataset = 100                 # the number of data sets used for calibration 

all_luconc <-NULL                 # dataframe to contain conctration by land use for all runs
all_site <-NULL                   # dataframe to contain concentration by calibration watershed for all runs

for (n in 1: num_dataset) {
  sampconc <-NULL
  for (i in 1:num_WS) {
    x<-rlnorm(1, meanlog[i], sdlog[i])
    oneconc <-data.frame(location=rownames(mean)[i], x, row.names=NULL)
    sampconc <- rbind(sampconc, oneconc)
  }
  obconc <-as.matrix(sampconc[,2]); rownames(obconc) <- sampconc$location;
  
  cat("x=", x, "obconc= ", obconc, "\n")
  
#####OPTIMIZATION PROCEDURE FOR ONE SET OF OBSERVED DATA####################################################

### set_up important constants for Box method
  step = 1.3                # constant determines how far to overshoot centroid
  nr = 4                    # constant for adjusting repeated unimprovements
  num_set = 2*num_LU        # parameter set generated =2*n
  max_gen = 1000            # number of generation
  gen = 1

###initialize matrix
  Hgconc <-matrix(0,nrow=num_set,ncol=num_LU)
  finalset <-matrix(0,nrow=num_set,ncol=num_LU)
  f <- matrix(0,nrow=1,ncol=num_set)

### Randomly generate the First Set of Parameters 
  for (i in 1:num_set) {
    for (j in 1:num_LU)  {
      Hgconc[i,j] <-lowbound[j] + (upbound[j]-lowbound[j])*runif(1)
    }
  }

### define objective function
  obf<- function (conc) {
    conc <-as.matrix(conc)
    Hgload <- runoff%*%conc                       #calculate Hg loads for each watershed based on runoff/concentration
    WSrunoff<-as.matrix(rowSums(runoff))          #sum of runoff for each watershed 
    WSconc <- Hgload/WSrunoff                     #calculate Hg concnetration at watershed outlet
    diff <-t(weight)%*%(WSconc-obconc)^2          #difference between simulated and observed concentration at each watershed 
    return(diff)
  }

### Calculate objective function for first set of parameters
  for (i in 1:num_set) {
    f[i]<- obf(Hgconc[i,])          # calculate objective function 
  }

### Loop to find best fit 
  while(gen<max_gen) {
    bestset <-which.min(f)          #find the best and worst parameter set
    bestf <-f[bestset]
    worstset<-which.max(f)
    worstf<-f[worstset]

    tempsets<-Hgconc[-worstset,]    #remove worst parameter set 
    centroid <-colSums(tempsets)/(num_set-1)
    newset <-centroid +step*(centroid-Hgconc[worstset,])
  
    for (i in 1:num_LU) {                      # check if newratio is within bound
      newset[i]<-ifelse(newset[i]<lowbound[i], lowbound[i], newset[i])
      newset[i]<-ifelse(newset[i]>upbound[i], upbound[i], newset[i])
    }

    newf <- obf(newset)
     
### check if new object is even worst than old one    
    if(newf > worstf) {
      kr<-1
      while (newf>worstf && gen<max_gen) {
        epsilon<-(nr/(nr+kr-1))^((nr+kr-1)/nr)
        if (epsilon < 0.001) {  
          break                                   # force termination for excessive loop  
        }
        kr<-kr+1
        rand<-runif(1)
        newset<-(newset+epsilon*centroid+(1-epsilon)*Hgconc[bestset,])/(2+(centroid-Hgconc[bestset,])*(1-epsilon)*(2*rand-1))
    
        for (i in 1:num_LU) {                      # check if newratio is within bound
          newset[i]<-ifelse(newset[i]<lowbound[i], lowbound[i], newset[i])
          newset[i]<-ifelse(newset[i]>upbound[i], upbound[i], newset[i])
        }
        newf<-obf(newset)   
      }
    } 
  
### creat new parameter matrix and calculate new objective function
    Hgconc<-rbind(tempsets, newset, deparse.level=0)
    for (i in 1:num_set) {
      f[i]<- obf(Hgconc[i,])  # calculate objective function 
    }   
    gen<-gen+1                   # increment iteration, loop again
  }

### record the final calibration results
  bestfit <- bestf
  finalset<- Hgconc   # record the final vertex
  optimconc <- newset

  simHgload <- runoff%*%optimconc                 
  simconc <- simHgload/runofftot
  simHgload <- simHgload*10^(-6)                        # conversion = 1000L*10^(-9)g/L =10^(-6)g
    
### put all land use ratios into one dataframe 
  luconc <- data.frame(landuse=colnames(concbound)[2:ncol(concbound)],n, optimconc)
  all_luconc <-rbind(all_luconc, luconc) 

### put all watershed ratios into one dataframe
  onesite <- data.frame(location=rownames(simconc), n, obconc, simconc, simHgload, row.names=NULL)
  all_site <- rbind(all_site, onesite) 

}   # end of loop of observed data points 

### compute mean and median of simulated conc/load
watershed_mean <- ddply(all_site, .(location), summarize, obmean = mean(obconc, na.rm = TRUE),                                        
                   simean = mean(simconc, na.rm = TRUE), 
                   loadmean= mean(simHgload, na.rm = TRUE))

watershed_median <- ddply(all_site, .(location), summarize, obmedian = median(obconc, na.rm = TRUE),                                        
                     simedian = median(simconc, na.rm = TRUE), 
                     loadmedian= median(simHgload, na.rm = TRUE))            

sumstats<- merge(watershed_mean, watershed_median, by = 'location')

quant <- ddply(all_luconc, .(landuse), summarize, quant0 = quantile(optimconc,0),
               quant25 = quantile(optimconc,0.25, na.rm = TRUE),
               quant50 = median(optimconc,na.rm = TRUE), 
               quant75 = quantile(optimconc, 0.75,na.rm = TRUE),
               quant100 = quantile(optimconc, 1,na.rm = TRUE))

### plot optimization results
setwd('C:/Projects/RWSM/2016 Work/Hg/Model results');

p0 <- ggplot(data=all_luconc, aes(x=landuse, y=optimconc)) + geom_boxplot(fill="orange") 
p0 <- p0 + theme_bw() + theme(legend.position = "none") 
p0 <- p0 + theme(axis.title.x = element_blank(), axis.text.x  = element_text(size=20)) + ylab("Hg concentration (ng/L)")
p0 <- p0 + theme(axis.title.y = element_text(face="bold", colour="black", size=20), axis.text.y=element_text(size=20)) + ylab('Hg Conc (ng/L)\n')
jpeg(paste('Boxplots for calibrated Hg conc by land use.jpg', sep=''), height=600, width=800)
print(p0)
dev.off()

p1 <- ggplot() + geom_boxplot(data=all_site, aes(x=location, y=simconc), fill="palegreen") 
p1 <- p1 + theme_bw() + theme(legend.position = "none") 
p1 <- p1 + theme(axis.title.x = element_blank(), axis.text.x  = element_text(angle=90, vjust=0.5, size=10, color='black')) 
p1 <- p1 + ylab('Hg Conc (ng/L)\n') 
plot <- p1+ geom_point(data=obHg, aes(x=Watershed,y=median), size=3, shape=23, fill='magenta1')
jpeg(paste('Boxplots for calibrated Hg conc to median.jpg', sep=''), height=400, width=800)
print(plot)
dev.off()

p2 <- ggplot() + geom_boxplot(data=all_site, aes(x=location, y=simconc), fill="palegreen")
p2 <- p2 + theme_bw() + theme(legend.position = "none")
p2 <- p2 + theme(axis.title.x = element_blank(), axis.text.x  = element_text(angle=90, vjust=0.5, size=10, color='black')) 
p2 <- p2 + ylab('Hg Conc (ng/L)\n') 
plot <- p2+ geom_point(data=obHg, aes(x=Watershed,y=mean), size=3, shape=23, fill='magenta1')
jpeg(paste('Boxplots for calibrated Hg conc to mean.jpg', sep=''), height=400, width=800)
print(plot)
dev.off()

p3 <- ggplot() + geom_boxplot(data=all_site, aes(x=location, y=simHgload), fill="palegreen") 
p3 <- p3 + theme_bw() + theme(legend.position = "none") 
p3 <- p3 + theme(axis.title.x = element_blank(), axis.text.x=element_text(angle=90, vjust=0.5, size=10, color='black')) 
p3 <- p3 + ylab('Hg loads (g)\n') 
jpeg(paste('Water based Hg loads at calibration watersheds.jpg', sep=''), height=400, width=800)
print(p3)
dev.off()

### put final results into a file
write.csv(all_luconc, file='Calibrated_Hg_concentration_by_landuse.csv');
write.csv(all_site, file='Observed_calibrated_Hg_concentration_by_watershed.csv');
write.csv(sumstats, file='Mean_median_by_watershed.csv');
write.csv(quant, file='quanticle_conc_by_landuse.csv')